/*
 * my_gps.c
 *
 *  Created on: 11 de mar de 2023
 *      Author: arne
 */

//******************************************************************************
// Arquivos incluídos no módulo
//******************************************************************************
#include "my_gps.h"

//******************************************************************************
// Variáveis externas do módulo
//******************************************************************************
extern char recbuf[TAMBUF];
extern char myHour[10];
extern char p[10];

extern volatile uint8_t state_mach;
extern volatile uint8_t counter;
extern volatile uint8_t autoriza;

//******************************************************************************
// Constantes externas do módulo
//******************************************************************************
extern const char gps_header1[5];

//*****************************************************************************
// Função GPS_getTime - extrai hora da string NMEA GPGGA
//
// $GPGGA, 161229.487, 3723.2475, N, 12158.3416, W, 1, 07, 1.0, 9.0, M, , , , 0000*18
//
// Message ID 	            | $GPGGA 	                                | GGA protocol header
// UTC time 	            | 161229.487 	                            | hhmmss.sss
// Latitude 	            | 3723.2475 (37 degrees, 23.2475 minutes)   |	ddmm.mmmm
// N/S Indicator 	        | N 	                                    | N = North, S = South
// Longitude 	            | 12158.3416 (121 degrees, 58.3416 minutes) | dddmm.mmmm
// E/W indicator 	        | W 	                                    | E = East or W = West
// Position Fix Indicator   | 1                                         |
// Satellites used 	        | 07 	                                    | Range is 0 to 12
// HDOP 	                | 1.0 	                                    | Horizontal Dilution of Precision
// MSL Altitude 	        | 9.0 	                                    | Meters
// Units 	                | M 	                                    | Meters
// Geoid Separation 		|                                           | Meters
// Units 	                | M 	                                    | Meters
// Age of diff. corr. 		|                                           | Second
// Diff. ref. station ID 	| 0000                                      |
// Checksum 	            | *18								        |
// <CR><LF> 		        |                                           | End of message termination
//
// Entradas - dado vindo do GPS e buffer a inserir a hora
// Saídas   - nenhuma (dados salvo no buffer)
//*****************************************************************************
uint8_t GPS_getTime(char NMEA[], char hora[]){

	uint8_t lret = 0;
	uint8_t aux = 0;

	if(NMEA[7] != ',' && NMEA[8] !=','){
		p[0] = NMEA[7];
		p[1] = NMEA[8];
		p[2] = '\0';

		aux = (uint8_t)atoi(p);
		if(aux >= 3 && aux <= 23){	//dentro da faixa
			aux -= 3;				//
		}
		else if(aux >= 0 && aux <= 2){//exige adaptação
			aux += 21;
		}

		sprintf(p, "%02d", aux);

		hora[0] = p[0];
		hora[1] = p[1];
		hora[2] = ':';
		hora[3] = NMEA[9];
		hora[4] = NMEA[10];
		hora[5] = ':';
		hora[6] = NMEA[11];
		hora[7] = NMEA[12];
		hora[8] = '\0';
		lret = 1;
	}
	else{
		hora[0] = '0';
		hora[1] = '0';
		hora[2] = ':';
		hora[3] = '0';
		hora[4] = '0';
		hora[5] = ':';
		hora[6] = '0';
		hora[7] = '0';
		hora[8] = '\0';
	}

	return lret;
}

//*****************************************************************************
// Função GPS_get_data - pega dados do GPS
//
// Entradas - dado vindo do GPS
// Saídas   - nenhuma (dados salvo no buffer)
//*****************************************************************************
void GPS_get_data(char dado){

   switch(state_mach){                      //trata máquina de estados
      case 0x00:                            //espera "$" - início da mensagem
         if(dado == '$'){                   //é header?
            recbuf[counter++] = dado;       //guarda dado
            state_mach++;                   //move máquina de estados
         }
         else{                               //do contrário, aguarda...
            counter = 0x00;                  //mantem ponteiro estacionado
         }
         break;
      case 0x01:                            //aguarda cabeçalho GPGGA
         if(dado == gps_header1[counter-1]){//se bate
            recbuf[counter++] = dado;       //guarda
            if((counter-1)==5)              //já pegou todo cabeçalho?
                state_mach++;               //incrementa máquina de estado
         }
         else{                              //do contrário
            counter = 0x00;                 //zera tudo e volta
            state_mach = 0x00;
         }
         break;
      case 0x02:                            //colhe dados até
         if(dado != CR){                 	//encontrar ENTER
            recbuf[counter++] = dado;       //guarda se não é ENTER
            if(counter >= TAMBUF){          //protege exceder tamanho máx. do buffer
                counter = 0x00;             //zera ponteiro
                state_mach = 0x00;
            }
         }
         else{
            recbuf[counter++] = CR;          //insere enter
            if(GPS_getTime(&recbuf[0], &myHour[0])) //pega a hora
            	autoriza = 1;
         }
         break;
      default:
         ;                             //
   }

}
